@@ -14,6 +14,7 @@ from bleak.backends.device import BLEDevice
 from bleak.backends.service import BleakGATTCharacteristic, BleakGATTServiceCollection
 from bleak.exc import BleakDBusError
 from bleak_retry_connector import (
+    BLEAK_RETRY_EXCEPTIONS,
     BleakClientWithServiceCache,
     BleakNotFoundError,
     ble_device_has_changed,
@@ -37,17 +38,11 @@ DEVICE_SET_EXTENDED_KEY = REQ_HEADER
 # Base key when encryption is set
 KEY_PASSWORD_PREFIX = "571"
 
-BLEAK_EXCEPTIONS = (
-    AttributeError,
-    BleakError,
-    EOFError,
-    asyncio.exceptions.TimeoutError,
-)
 
 # How long to hold the connection
 # to wait for additional commands for
 # disconnecting the device.
-DISCONNECT_DELAY = 49
+DISCONNECT_DELAY = 20
 
 
 class ColorMode(Enum):
@@ -107,13 +102,13 @@ class SwitchbotBaseDevice:
                 binascii.crc32(password.encode("ascii")) & 0xFFFFFFFF
             )
         self._client: BleakClientWithServiceCache | None = None
-        self._cached_services: BleakGATTServiceCollection | None = None
         self._read_char: BleakGATTCharacteristic | None = None
         self._write_char: BleakGATTCharacteristic | None = None
         self._disconnect_timer: asyncio.TimerHandle | None = None
         self._expected_disconnect = False
         self.loop = asyncio.get_event_loop()
         self._callbacks: list[Callable[[], None]] = []
+        self._notify_future: asyncio.Future[bytearray] | None = None
 
     def advertisement_changed(self, advertisement: SwitchBotAdvertisement) -> bool:
         """Check if the advertisement has changed."""
@@ -136,14 +131,7 @@ class SwitchbotBaseDevice:
         if retry is None:
             retry = self._retry_count
         command = bytearray.fromhex(self._commandkey(key))
-        _LOGGER.debug("%s: Sending command %s", self.name, command)
-        if self._operation_lock.locked():
-            _LOGGER.debug(
-                "%s: Operation already in progress, waiting for it to complete; RSSI: %s",
-                self.name,
-                self.rssi,
-            )
-
+        _LOGGER.debug("%s: Scheduling command %s", self.name, command.hex())
         max_attempts = retry + 1
         if self._operation_lock.locked():
             _LOGGER.debug(
@@ -181,7 +169,7 @@ class SwitchbotBaseDevice:
                         self.rssi,
                         exc_info=True,
                     )
-                except BLEAK_EXCEPTIONS:
+                except BLEAK_RETRY_EXCEPTIONS:
                     if attempt == retry:
                         _LOGGER.error(
                             "%s: communication failed; Stopping trying; RSSI: %s",
@@ -229,7 +217,7 @@ class SwitchbotBaseDevice:
                 self._device,
                 self.name,
                 self._disconnected,
-                cached_services=self._cached_services,
+                use_services_cache=True,
                 ble_device_callback=lambda: self._device,
             )
             _LOGGER.debug("%s: Connected; RSSI: %s", self.name, self.rssi)
@@ -237,9 +225,9 @@ class SwitchbotBaseDevice:
             if not resolved:
                 # Try to handle services failing to load
                 resolved = self._resolve_characteristics(await client.get_services())
-            self._cached_services = client.services if resolved else None
             self._client = client
             self._reset_disconnect_timer()
+            await self._start_notify()
 
     def _resolve_characteristics(self, services: BleakGATTServiceCollection) -> bool:
         """Resolve characteristics."""
@@ -337,6 +325,18 @@ class SwitchbotBaseDevice:
             await self._execute_forced_disconnect()
             raise
 
+    def _notification_handler(self, _sender: int, data: bytearray) -> None:
+        """Handle notification responses."""
+        if self._notify_future and not self._notify_future.done():
+            self._notify_future.set_result(data)
+            return
+        _LOGGER.debug("%s: Received unsolicited notification: %s", self.name, data)
+
+    async def _start_notify(self) -> None:
+        """Start notification."""
+        _LOGGER.debug("%s: Subscribe to notifications; RSSI: %s", self.name, self.rssi)
+        await self._client.start_notify(self._read_char, self._notification_handler)
+
     async def _execute_command_locked(self, key: str, command: bytes) -> bytes:
         """Execute command and read response."""
         assert self._client is not None
@@ -344,28 +344,16 @@ class SwitchbotBaseDevice:
             raise CharacteristicMissingError(READ_CHAR_UUID)
         if not self._write_char:
             raise CharacteristicMissingError(WRITE_CHAR_UUID)
-        future: asyncio.Future[bytearray] = asyncio.Future()
+        self._notify_future = asyncio.Future()
         client = self._client
 
-        def _notification_handler(_sender: int, data: bytearray) -> None:
-            """Handle notification responses."""
-            if future.done():
-                _LOGGER.debug("%s: Notification handler already done", self.name)
-                return
-            future.set_result(data)
-
-        _LOGGER.debug("%s: Subscribe to notifications; RSSI: %s", self.name, self.rssi)
-        await client.start_notify(self._read_char, _notification_handler)
-
         _LOGGER.debug("%s: Sending command: %s", self.name, key)
         await client.write_gatt_char(self._write_char, command, False)
 
         async with async_timeout.timeout(5):
-            notify_msg = await future
-        _LOGGER.debug("%s: Notification received: %s", self.name, notify_msg)
-
-        _LOGGER.debug("%s: UnSubscribe to notifications", self.name)
-        await client.stop_notify(self._read_char)
+            notify_msg = await self._notify_future
+        _LOGGER.debug("%s: Notification received: %s", self.name, notify_msg.hex())
+        self._notify_future = None
 
         if notify_msg == b"\x07":
             _LOGGER.error("Password required")
@@ -399,8 +387,6 @@ class SwitchbotBaseDevice:
         """Update device data from advertisement."""
         # Only accept advertisements if the data is not missing
         # if we already have an advertisement with data
-        if self._device and ble_device_has_changed(self._device, advertisement.device):
-            self._cached_services = None
         self._device = advertisement.device
 
     async def get_device_data(
